@@ -41,10 +41,10 @@ G_BEGIN_DECLS
 /**
  * GstBinFlags:
  * @GST_BIN_FLAG_NO_RESYNC: don't resync a state change when elements are
- *             added or linked in the bin (Since 1.0.5)
+ *             added or linked in the bin (Since: 1.0.5)
  * @GST_BIN_FLAG_STREAMS_AWARE: Indicates whether the bin can handle elements
  *             that add/remove source pads at any point in time without
- *             first posting a no-more-pads signal (Since 1.10)
+ *             first posting a no-more-pads signal (Since: 1.10)
  * @GST_BIN_FLAG_LAST: the last enum in the series of flags for bins.
  * Derived classes can use this as first value in a list of flags.
  *
@@ -192,67 +192,68 @@ struct _GstBinClass {
   gpointer _gst_reserved[GST_PADDING-2];
 };
 
-GST_EXPORT
+GST_API
 GType		gst_bin_get_type		(void);
 
-GST_EXPORT
+GST_API
 GstElement*	gst_bin_new			(const gchar *name);
 
 /* add and remove elements from the bin */
 
-GST_EXPORT
+GST_API
 gboolean	gst_bin_add			(GstBin *bin, GstElement *element);
 
-GST_EXPORT
+GST_API
 gboolean	gst_bin_remove			(GstBin *bin, GstElement *element);
 
 /* retrieve a single child */
 
-GST_EXPORT
+GST_API
 GstElement*	gst_bin_get_by_name		 (GstBin *bin, const gchar *name);
 
-GST_EXPORT
+GST_API
 GstElement*	gst_bin_get_by_name_recurse_up	 (GstBin *bin, const gchar *name);
 
-GST_EXPORT
+GST_API
 GstElement*	gst_bin_get_by_interface	 (GstBin *bin, GType iface);
 
 /* retrieve multiple children */
 
-GST_EXPORT
+GST_API
 GstIterator*    gst_bin_iterate_elements	 (GstBin *bin);
 
-GST_EXPORT
+GST_API
 GstIterator*    gst_bin_iterate_sorted		 (GstBin *bin);
 
-GST_EXPORT
+GST_API
 GstIterator*    gst_bin_iterate_recurse		 (GstBin *bin);
 
-GST_EXPORT
+GST_API
 GstIterator*	gst_bin_iterate_sinks		 (GstBin *bin);
 
-GST_EXPORT
+GST_API
 GstIterator*	gst_bin_iterate_sources		 (GstBin *bin);
 
-GST_EXPORT
+GST_API
 GstIterator*	gst_bin_iterate_all_by_interface (GstBin *bin, GType iface);
 
+GST_API
+GstIterator*	gst_bin_iterate_all_by_element_factory_name (GstBin * bin, const gchar * factory_name);
+
 /* latency */
 
-GST_EXPORT
+GST_API
 gboolean        gst_bin_recalculate_latency      (GstBin * bin);
 
 /* set and get suppressed flags */
 
-GST_EXPORT
+GST_API
 void            gst_bin_set_suppressed_flags (GstBin * bin, GstElementFlags flags);
 
-GST_EXPORT
+GST_API
 GstElementFlags gst_bin_get_suppressed_flags (GstBin * bin);
 
-#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstBin, gst_object_unref)
-#endif
 
 G_END_DECLS
 
