@@ -177,12 +177,17 @@ typedef enum { /*< flags >*/
 
 /**
  * GstStackTraceFlags:
- * @GST_STACK_TRACE_SHOW_FULL: Try to retrieve as much information as
- *                             possible when getting the stack trace
+ * @GST_STACK_TRACE_SHOW_NONE: Try to retrieve the minimum information
+ *                             available, which may be none on some platforms
+ *                             (Since: 1.18)
+ * @GST_STACK_TRACE_SHOW_FULL: Try to retrieve as much information as possible,
+ *                             including source information when getting the
+ *                             stack trace
  *
  * Since: 1.12
  */
 typedef enum {
+    GST_STACK_TRACE_SHOW_NONE = 0,
     GST_STACK_TRACE_SHOW_FULL = 1 << 0
 } GstStackTraceFlags;
 
@@ -253,7 +258,7 @@ struct _GstDebugCategory {
  * GST_FUNCTION:
  *
  * This macro should evaluate to the name of the current function and be should
- * be defined when configuring your project, as it is compiler dependant. If it
+ * be defined when configuring your project, as it is compiler dependent. If it
  * is not defined, some default value is used. It is used to provide debugging
  * output with the function name of the message.
  *
@@ -271,7 +276,7 @@ struct _GstDebugCategory {
 #endif /* ifndef GST_FUNCTION */
 
 /**
- * GST_PTR_FORMAT:
+ * GST_PTR_FORMAT: (skip):
  *
  * printf format type used to debug GStreamer types. You can use this in
  * combination with GStreamer's debug logging system as well as the functions
@@ -287,7 +292,7 @@ struct _GstDebugCategory {
 #define GST_PTR_FORMAT     "p\aA"
 
 /**
- * GST_SEGMENT_FORMAT:
+ * GST_SEGMENT_FORMAT: (skip):
  *
  * printf format type used to debug GStreamer segments. You can use this in
  * combination with GStreamer's debug logging system as well as the functions
@@ -323,7 +328,7 @@ typedef void (*GstLogFunction)  (GstDebugCategory * category,
                                  GstDebugMessage  * message,
                                  gpointer           user_data);
 
-GST_EXPORT
+GST_API
 void		    gst_debug_log            (GstDebugCategory * category,
                                           GstDebugLevel      level,
                                           const gchar      * file,
@@ -332,7 +337,7 @@ void		    gst_debug_log            (GstDebugCategory * category,
                                           GObject          * object,
                                           const gchar      * format,
                                           ...) G_GNUC_PRINTF (7, 8) G_GNUC_NO_INSTRUMENT;
-GST_EXPORT
+GST_API
 void            gst_debug_log_valist     (GstDebugCategory * category,
                                           GstDebugLevel      level,
                                           const gchar      * file,
@@ -344,25 +349,27 @@ void            gst_debug_log_valist     (GstDebugCategory * category,
 
 /* do not use this function, use the GST_DEBUG_CATEGORY_INIT macro */
 
-GST_EXPORT
+GST_API
 GstDebugCategory *_gst_debug_category_new (const gchar * name,
                                            guint         color,
                                            const gchar * description);
 
 /* do not use this function, use the GST_DEBUG_CATEGORY_GET macro */
 
-GST_EXPORT
+GST_API
 GstDebugCategory *_gst_debug_get_category (const gchar *name);
 
 
 /* do not use this function, use the GST_CAT_MEMDUMP_* macros */
 
-GST_EXPORT
+GST_API
 void _gst_debug_dump_mem (GstDebugCategory * cat, const gchar * file,
     const gchar * func, gint line, GObject * obj, const gchar * msg,
     const guint8 * data, guint length);
 
-/* we define this to avoid a compiler warning regarding a cast from a function
+/**
+ * GstDebugFuncPtr: (attributes doc.skip=true)
+ * we define this to avoid a compiler warning regarding a cast from a function
  * pointer to a void pointer
  * (see https://bugzilla.gnome.org/show_bug.cgi?id=309253)
  */
@@ -370,18 +377,27 @@ typedef	void (* GstDebugFuncPtr)	(void);
 
 /* do no use these functions, use the GST_DEBUG*_FUNCPTR macros */
 
-GST_EXPORT
+GST_API
 void	_gst_debug_register_funcptr	(GstDebugFuncPtr	func,
 					 const gchar *		ptrname);
-GST_EXPORT
+GST_API
 const gchar *
 	_gst_debug_nameof_funcptr	(GstDebugFuncPtr	func) G_GNUC_NO_INSTRUMENT;
 
 
-GST_EXPORT
+GST_API
 const gchar   * gst_debug_message_get    (GstDebugMessage  * message);
 
-GST_EXPORT
+GST_API
+gchar         * gst_debug_log_get_line    (GstDebugCategory * category,
+                                           GstDebugLevel      level,
+                                           const gchar      * file,
+                                           const gchar      * function,
+                                           gint               line,
+                                           GObject          * object,
+                                           GstDebugMessage  * message) G_GNUC_NO_INSTRUMENT;
+
+GST_API
 void            gst_debug_log_default    (GstDebugCategory * category,
                                           GstDebugLevel      level,
                                           const gchar      * file,
@@ -390,108 +406,108 @@ void            gst_debug_log_default    (GstDebugCategory * category,
                                           GObject          * object,
                                           GstDebugMessage  * message,
                                           gpointer           user_data) G_GNUC_NO_INSTRUMENT;
-GST_EXPORT
+
+GST_API
 const gchar *   gst_debug_level_get_name (GstDebugLevel level);
 
-GST_EXPORT
+GST_API
 void            gst_debug_add_log_function            (GstLogFunction func,
                                                        gpointer       user_data,
                                                        GDestroyNotify notify);
-GST_EXPORT
+GST_API
 guint           gst_debug_remove_log_function         (GstLogFunction func);
 
-GST_EXPORT
+GST_API
 guint           gst_debug_remove_log_function_by_data (gpointer       data);
 
-GST_EXPORT
+GST_API
 void            gst_debug_set_active  (gboolean active);
 
-GST_EXPORT
+GST_API
 gboolean        gst_debug_is_active   (void);
 
-GST_EXPORT
+GST_API
 void            gst_debug_set_colored (gboolean colored);
 
-GST_EXPORT
+GST_API
 void            gst_debug_set_color_mode   (GstDebugColorMode mode);
 
-GST_EXPORT
+GST_API
 void            gst_debug_set_color_mode_from_string (const gchar * mode);
 
-GST_EXPORT
+GST_API
 gboolean        gst_debug_is_colored  (void);
 
-GST_EXPORT
+GST_API
 GstDebugColorMode gst_debug_get_color_mode (void);
 
-GST_EXPORT
+GST_API
 void            gst_debug_set_default_threshold      (GstDebugLevel level);
 
-GST_EXPORT
+GST_API
 GstDebugLevel   gst_debug_get_default_threshold      (void);
 
-GST_EXPORT
+GST_API
 void            gst_debug_set_threshold_for_name     (const gchar * name,
                                                       GstDebugLevel level);
-GST_EXPORT
+GST_API
 void            gst_debug_set_threshold_from_string  (const gchar * list, gboolean reset);
 
-GST_EXPORT
+GST_API
 void            gst_debug_unset_threshold_for_name   (const gchar * name);
 
-
-GST_EXPORT
+GST_DEPRECATED
 void            gst_debug_category_free              (GstDebugCategory *	category);
 
-GST_EXPORT
+GST_API
 void            gst_debug_category_set_threshold     (GstDebugCategory *	category,
                                                       GstDebugLevel		level);
 
-GST_EXPORT
+GST_API
 void            gst_debug_category_reset_threshold   (GstDebugCategory *	category);
 
-GST_EXPORT
+GST_API
 GstDebugLevel   gst_debug_category_get_threshold     (GstDebugCategory *	category);
 
-GST_EXPORT
+GST_API
 const gchar *   gst_debug_category_get_name          (GstDebugCategory *	category);
 
-GST_EXPORT
+GST_API
 guint           gst_debug_category_get_color         (GstDebugCategory *	category);
 
-GST_EXPORT
+GST_API
 const gchar *   gst_debug_category_get_description   (GstDebugCategory *	category);
 
-GST_EXPORT
+GST_API
 GSList *        gst_debug_get_all_categories	(void);
 
 
-GST_EXPORT
+GST_API
 gchar * gst_debug_construct_term_color (guint colorinfo);
 
-GST_EXPORT
+GST_API
 gint    gst_debug_construct_win_color  (guint colorinfo);
 
-GST_EXPORT
+GST_API
 gint    gst_info_vasprintf              (gchar ** result,
                                          const gchar * format,
                                          va_list args) G_GNUC_PRINTF (2, 0);
-GST_EXPORT
+GST_API
 gchar * gst_info_strdup_vprintf         (const gchar *format, va_list args) G_GNUC_PRINTF (1, 0);
 
-GST_EXPORT
+GST_API
 gchar * gst_info_strdup_printf          (const gchar *format, ...) G_GNUC_PRINTF (1, 2);
 
-GST_EXPORT
+GST_API
 void    gst_print                       (const gchar * format, ...) G_GNUC_PRINTF (1, 2);
 
-GST_EXPORT
+GST_API
 void    gst_println                     (const gchar * format, ...) G_GNUC_PRINTF (1, 2);
 
-GST_EXPORT
+GST_API
 void    gst_printerr                    (const gchar * format, ...) G_GNUC_PRINTF (1, 2);
 
-GST_EXPORT
+GST_API
 void    gst_printerrln                  (const gchar * format, ...) G_GNUC_PRINTF (1, 2);
 
 #ifndef GST_DISABLE_GST_DEBUG
@@ -609,15 +625,15 @@ G_STMT_START{                                        \
  * Default gstreamer core debug log category. Please define your own.
  */
 
-GST_EXPORT GstDebugCategory *	GST_CAT_DEFAULT;
+GST_API GstDebugCategory *	GST_CAT_DEFAULT;
 /* this symbol may not be used */
 
-GST_EXPORT gboolean                 _gst_debug_enabled;
+GST_API gboolean                 _gst_debug_enabled;
 
 /* the min debug level, used for quickly discarding debug
  * messages that fall under the threshold. */
 
-GST_EXPORT GstDebugLevel            _gst_debug_min;
+GST_API GstDebugLevel            _gst_debug_min;
 
 /**
  * GST_CAT_LEVEL_LOG:
@@ -635,7 +651,8 @@ GST_EXPORT GstDebugLevel            _gst_debug_min;
  */
 #ifdef G_HAVE_ISO_VARARGS
 #define GST_CAT_LEVEL_LOG(cat,level,object,...) G_STMT_START{		\
-  if (G_UNLIKELY ((level) <= GST_LEVEL_MAX && (level) <= _gst_debug_min)) {						\
+  if (G_UNLIKELY ((level) <= GST_LEVEL_MAX && (level) <= _gst_debug_min	\
+      && (level) <= gst_debug_category_get_threshold (cat))) {		\
     gst_debug_log ((cat), (level), __FILE__, GST_FUNCTION, __LINE__,	\
         (GObject *) (object), __VA_ARGS__);				\
   }									\
@@ -643,7 +660,8 @@ GST_EXPORT GstDebugLevel            _gst_debug_min;
 #else /* G_HAVE_GNUC_VARARGS */
 #ifdef G_HAVE_GNUC_VARARGS
 #define GST_CAT_LEVEL_LOG(cat,level,object,args...) G_STMT_START{	\
-  if (G_UNLIKELY ((level) <= GST_LEVEL_MAX && (level) <= _gst_debug_min)) {						\
+  if (G_UNLIKELY ((level) <= GST_LEVEL_MAX && (level) <= _gst_debug_min 	\
+      && (level) <= gst_debug_category_get_threshold (cat))) {		\
     gst_debug_log ((cat), (level), __FILE__, GST_FUNCTION, __LINE__,	\
         (GObject *) (object), ##args );					\
   }									\
@@ -653,7 +671,8 @@ static inline void
 GST_CAT_LEVEL_LOG_valist (GstDebugCategory * cat,
     GstDebugLevel level, gpointer object, const char *format, va_list varargs)
 {
-  if (G_UNLIKELY ((level) <= GST_LEVEL_MAX && (level) <= _gst_debug_min)) {
+  if (G_UNLIKELY ((level) <= GST_LEVEL_MAX && (level) <= _gst_debug_min
+      && (level) <= gst_debug_category_get_threshold (cat))) {
     gst_debug_log_valist (cat, level, "", "", 0, (GObject *) object, format,
         varargs);
   }
@@ -1485,7 +1504,9 @@ GST_TRACE (const char *format, ...)
 
 #define GST_DEBUG_CATEGORY_INIT(var,name,color,desc)	G_STMT_START{ }G_STMT_END
 #define GST_DEBUG_CATEGORY_GET(var,name)		G_STMT_START{ }G_STMT_END
+#ifndef GST_DISABLE_DEPRECATED
 #define gst_debug_category_free(category)		G_STMT_START{ }G_STMT_END
+#endif
 #define gst_debug_category_set_threshold(category,level) G_STMT_START{ }G_STMT_END
 #define gst_debug_category_reset_threshold(category)	G_STMT_START{ }G_STMT_END
 #define gst_debug_category_get_threshold(category)	(GST_LEVEL_NONE)
@@ -1740,17 +1761,17 @@ GST_TRACE (const char *format, ...)
 #endif /* GST_DISABLE_GST_DEBUG */
 
 
-GST_EXPORT
+GST_API
 void gst_debug_print_stack_trace (void);
 
-GST_EXPORT
+GST_API
 gchar * gst_debug_get_stack_trace (GstStackTraceFlags flags);
 
-GST_EXPORT
+GST_API
 void                  gst_debug_add_ring_buffer_logger      (guint max_size_per_thread, guint thread_timeout);
-GST_EXPORT
+GST_API
 void                  gst_debug_remove_ring_buffer_logger   (void);
-GST_EXPORT
+GST_API
 gchar **              gst_debug_ring_buffer_logger_get_logs (void);
 
 G_END_DECLS
