@@ -99,6 +99,7 @@ typedef enum {
  * @GST_QUERY_DRAIN: wait till all serialized data is consumed downstream
  * @GST_QUERY_CONTEXT: query the pipeline-local context from
  *     downstream or upstream (since 1.2)
+ * @GST_QUERY_BITRATE: the bitrate query (since 1.16)
  *
  * Standard predefined Query types
  */
@@ -123,11 +124,12 @@ typedef enum {
   GST_QUERY_ACCEPT_CAPS  = GST_QUERY_MAKE_TYPE (160, FLAG(BOTH)),
   GST_QUERY_CAPS         = GST_QUERY_MAKE_TYPE (170, FLAG(BOTH)),
   GST_QUERY_DRAIN        = GST_QUERY_MAKE_TYPE (180, FLAG(DOWNSTREAM) | FLAG(SERIALIZED)),
-  GST_QUERY_CONTEXT      = GST_QUERY_MAKE_TYPE (190, FLAG(BOTH))
+  GST_QUERY_CONTEXT      = GST_QUERY_MAKE_TYPE (190, FLAG(BOTH)),
+  GST_QUERY_BITRATE      = GST_QUERY_MAKE_TYPE (200, FLAG(DOWNSTREAM)),
 } GstQueryType;
 #undef FLAG
 
-GST_EXPORT GType _gst_query_type;
+GST_API GType _gst_query_type;
 
 #define GST_TYPE_QUERY                         (_gst_query_type)
 #define GST_IS_QUERY(obj)                      (GST_IS_MINI_OBJECT_TYPE (obj, GST_TYPE_QUERY))
@@ -212,18 +214,18 @@ typedef enum {
 #include <gst/gsttoc.h>
 #include <gst/gstcontext.h>
 
-GST_EXPORT
+GST_API
 const gchar*    gst_query_type_get_name        (GstQueryType type);
 
-GST_EXPORT
+GST_API
 GQuark          gst_query_type_to_quark        (GstQueryType type);
 
-GST_EXPORT
+GST_API
 GstQueryTypeFlags
                 gst_query_type_get_flags       (GstQueryType type);
 
 
-GST_EXPORT
+GST_API
 GType           gst_query_get_type             (void);
 
 /* refcounting */
@@ -242,21 +244,41 @@ gst_query_ref (GstQuery * q)
 }
 
 /**
- * gst_query_unref:
+ * gst_query_unref: (skip)
  * @q: a #GstQuery to decrease the refcount of.
  *
  * Decreases the refcount of the query. If the refcount reaches 0, the query
  * will be freed.
  */
+static inline void gst_query_unref(GstQuery* q);
 static inline void
 gst_query_unref (GstQuery * q)
 {
   gst_mini_object_unref (GST_MINI_OBJECT_CAST (q));
 }
 
+/**
+ * gst_clear_query: (skip)
+ * @query_ptr: a pointer to a #GstQuery reference
+ *
+ * Clears a reference to a #GstQuery.
+ *
+ * @query_ptr must not be %NULL.
+ *
+ * If the reference is %NULL then this function does nothing. Otherwise, the
+ * reference count of the query is decreased and the pointer is set to %NULL.
+ *
+ * Since: 1.16
+ */
+static inline void
+gst_clear_query (GstQuery ** query_ptr)
+{
+  gst_clear_mini_object ((GstMiniObject **) query_ptr);
+}
+
 /* copy query */
 /**
- * gst_query_copy:
+ * gst_query_copy: (skip)
  * @q: a #GstQuery to copy.
  *
  * Copies the given query using the copy function of the parent #GstStructure.
@@ -265,6 +287,7 @@ gst_query_unref (GstQuery * q)
  *
  * Returns: (transfer full): a new copy of @q.
  */
+static inline GstQuery* gst_query_copy(const GstQuery* q);
 static inline GstQuery *
 gst_query_copy (const GstQuery * q)
 {
@@ -288,7 +311,7 @@ gst_query_copy (const GstQuery * q)
  */
 #define         gst_query_make_writable(q)      GST_QUERY_CAST (gst_mini_object_make_writable (GST_MINI_OBJECT_CAST (q)))
 /**
- * gst_query_replace:
+ * gst_query_replace: (skip)
  * @old_query: (inout) (transfer full) (nullable): pointer to a pointer to a
  *     #GstQuery to be replaced.
  * @new_query: (allow-none) (transfer none): pointer to a #GstQuery that will
@@ -303,254 +326,279 @@ gst_query_copy (const GstQuery * q)
  *
  * Returns: %TRUE if @new_query was different from @old_query
  */
+static inline gboolean gst_query_replace(GstQuery** old_query, GstQuery* new_query);
 static inline gboolean
 gst_query_replace (GstQuery **old_query, GstQuery *new_query)
 {
   return gst_mini_object_replace ((GstMiniObject **) old_query, (GstMiniObject *) new_query);
 }
 
+/**
+ * gst_query_take:
+ * @old_query: (inout) (transfer full) (nullable): pointer to a
+ *     pointer to a #GstQuery to be stolen.
+ * @new_query: (allow-none) (transfer full): pointer to a #GstQuery that will
+ *     replace the query pointed to by @old_query.
+ *
+ * Modifies a pointer to a #GstQuery to point to a different #GstQuery. This
+ * function is similar to gst_query_replace() except that it takes ownership of
+ * @new_query.
+ *
+ * Either @new_query or the #GstQuery pointed to by @old_query may be %NULL.
+ *
+ * Returns: %TRUE if @new_query was different from @old_query
+ *
+ * Since: 1.16
+ */
+static inline gboolean
+gst_query_take (GstQuery **old_query, GstQuery *new_query)
+{
+  return gst_mini_object_take ((GstMiniObject **) old_query,
+      (GstMiniObject *) new_query);
+}
+
 /* application specific query */
 
-GST_EXPORT
+GST_API
 GstQuery *      gst_query_new_custom            (GstQueryType type, GstStructure *structure) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 const GstStructure *
                 gst_query_get_structure         (GstQuery *query);
 
-GST_EXPORT
+GST_API
 GstStructure *  gst_query_writable_structure    (GstQuery *query);
 
 /* position query */
 
-GST_EXPORT
+GST_API
 GstQuery*       gst_query_new_position          (GstFormat format) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_query_set_position          (GstQuery *query, GstFormat format, gint64 cur);
 
-GST_EXPORT
+GST_API
 void            gst_query_parse_position        (GstQuery *query, GstFormat *format, gint64 *cur);
 
 /* duration query */
 
-GST_EXPORT
+GST_API
 GstQuery*       gst_query_new_duration          (GstFormat format) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_query_set_duration          (GstQuery *query, GstFormat format, gint64 duration);
 
-GST_EXPORT
+GST_API
 void            gst_query_parse_duration        (GstQuery *query, GstFormat *format, gint64 *duration);
 
 /* latency query */
 
-GST_EXPORT
+GST_API
 GstQuery*       gst_query_new_latency           (void) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_query_set_latency           (GstQuery *query, gboolean live, GstClockTime min_latency,
                                                  GstClockTime max_latency);
 
-GST_EXPORT
+GST_API
 void            gst_query_parse_latency         (GstQuery *query, gboolean *live, GstClockTime *min_latency,
                                                  GstClockTime *max_latency);
 
 /* convert query */
 
-GST_EXPORT
+GST_API
 GstQuery*       gst_query_new_convert           (GstFormat src_format, gint64 value, GstFormat dest_format) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_query_set_convert           (GstQuery *query, GstFormat src_format, gint64 src_value,
                                                  GstFormat dest_format, gint64 dest_value);
 
-GST_EXPORT
+GST_API
 void            gst_query_parse_convert         (GstQuery *query, GstFormat *src_format, gint64 *src_value,
                                                  GstFormat *dest_format, gint64 *dest_value);
 /* segment query */
 
-GST_EXPORT
+GST_API
 GstQuery*       gst_query_new_segment           (GstFormat format) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_query_set_segment           (GstQuery *query, gdouble rate, GstFormat format,
                                                  gint64 start_value, gint64 stop_value);
 
-GST_EXPORT
+GST_API
 void            gst_query_parse_segment         (GstQuery *query, gdouble *rate, GstFormat *format,
                                                  gint64 *start_value, gint64 *stop_value);
 
 /* seeking query */
 
-GST_EXPORT
+GST_API
 GstQuery*       gst_query_new_seeking           (GstFormat format) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_query_set_seeking           (GstQuery *query, GstFormat format,
                                                  gboolean seekable,
                                                  gint64 segment_start,
                                                  gint64 segment_end);
 
-GST_EXPORT
+GST_API
 void            gst_query_parse_seeking         (GstQuery *query, GstFormat *format,
                                                  gboolean *seekable,
                                                  gint64 *segment_start,
                                                  gint64 *segment_end);
 /* formats query */
 
-GST_EXPORT
+GST_API
 GstQuery*       gst_query_new_formats           (void) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_query_set_formats           (GstQuery *query, gint n_formats, ...);
 
-GST_EXPORT
+GST_API
 void            gst_query_set_formatsv          (GstQuery *query, gint n_formats, const GstFormat *formats);
 
-GST_EXPORT
+GST_API
 void            gst_query_parse_n_formats       (GstQuery *query, guint *n_formats);
 
-GST_EXPORT
+GST_API
 void            gst_query_parse_nth_format      (GstQuery *query, guint nth, GstFormat *format);
 
 /* buffering query */
 
-GST_EXPORT
+GST_API
 GstQuery*       gst_query_new_buffering           (GstFormat format) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_query_set_buffering_percent   (GstQuery *query, gboolean busy, gint percent);
 
-GST_EXPORT
+GST_API
 void            gst_query_parse_buffering_percent (GstQuery *query, gboolean *busy, gint *percent);
 
-GST_EXPORT
+GST_API
 void            gst_query_set_buffering_stats     (GstQuery *query, GstBufferingMode mode,
                                                    gint avg_in, gint avg_out,
                                                    gint64 buffering_left);
 
-GST_EXPORT
+GST_API
 void            gst_query_parse_buffering_stats    (GstQuery *query, GstBufferingMode *mode,
                                                    gint *avg_in, gint *avg_out,
                                                    gint64 *buffering_left);
 
-GST_EXPORT
+GST_API
 void            gst_query_set_buffering_range     (GstQuery *query, GstFormat format,
                                                    gint64 start, gint64 stop,
                                                    gint64 estimated_total);
 
-GST_EXPORT
+GST_API
 void            gst_query_parse_buffering_range   (GstQuery *query, GstFormat *format,
                                                    gint64 *start, gint64 *stop,
                                                    gint64 *estimated_total);
 
-GST_EXPORT
+GST_API
 gboolean        gst_query_add_buffering_range       (GstQuery *query,
                                                      gint64 start, gint64 stop);
 
-GST_EXPORT
+GST_API
 guint           gst_query_get_n_buffering_ranges    (GstQuery *query);
 
-GST_EXPORT
+GST_API
 gboolean        gst_query_parse_nth_buffering_range (GstQuery *query,
                                                      guint index, gint64 *start,
                                                      gint64 *stop);
 
 /* URI query */
 
-GST_EXPORT
+GST_API
 GstQuery *      gst_query_new_uri                    (void) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_query_parse_uri                  (GstQuery *query, gchar **uri);
 
-GST_EXPORT
+GST_API
 void            gst_query_set_uri                    (GstQuery *query, const gchar *uri);
 
-GST_EXPORT
+GST_API
 void            gst_query_parse_uri_redirection      (GstQuery *query, gchar **uri);
 
-GST_EXPORT
+GST_API
 void            gst_query_set_uri_redirection        (GstQuery *query, const gchar *uri);
 
-GST_EXPORT
+GST_API
 void            gst_query_parse_uri_redirection_permanent (GstQuery *query, gboolean * permanent);
 
-GST_EXPORT
+GST_API
 void            gst_query_set_uri_redirection_permanent (GstQuery *query, gboolean permanent);
 
 /* allocation query */
 
-GST_EXPORT
+GST_API
 GstQuery *      gst_query_new_allocation             (GstCaps *caps, gboolean need_pool) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_query_parse_allocation           (GstQuery *query, GstCaps **caps, gboolean *need_pool);
 
 /* pools */
 
-GST_EXPORT
+GST_API
 void            gst_query_add_allocation_pool        (GstQuery *query, GstBufferPool *pool,
                                                       guint size, guint min_buffers,
                                                       guint max_buffers);
 
-GST_EXPORT
+GST_API
 guint           gst_query_get_n_allocation_pools     (GstQuery *query);
 
-GST_EXPORT
+GST_API
 void            gst_query_parse_nth_allocation_pool  (GstQuery *query, guint index,
                                                       GstBufferPool **pool,
                                                       guint *size, guint *min_buffers,
                                                       guint *max_buffers);
 
-GST_EXPORT
+GST_API
 void            gst_query_set_nth_allocation_pool    (GstQuery *query, guint index,
                                                       GstBufferPool *pool,
                                                       guint size, guint min_buffers,
                                                       guint max_buffers);
 
-GST_EXPORT
+GST_API
 void            gst_query_remove_nth_allocation_pool (GstQuery *query, guint index);
 
 /* allocators */
 
-GST_EXPORT
+GST_API
 void            gst_query_add_allocation_param       (GstQuery *query, GstAllocator *allocator,
                                                       const GstAllocationParams *params);
 
-GST_EXPORT
+GST_API
 guint           gst_query_get_n_allocation_params    (GstQuery *query);
 
-GST_EXPORT
+GST_API
 void            gst_query_parse_nth_allocation_param (GstQuery *query, guint index,
                                                       GstAllocator **allocator,
                                                       GstAllocationParams *params);
 
-GST_EXPORT
+GST_API
 void            gst_query_set_nth_allocation_param   (GstQuery *query, guint index,
                                                       GstAllocator *allocator,
                                                       const GstAllocationParams *params);
 
-GST_EXPORT
+GST_API
 void            gst_query_remove_nth_allocation_param (GstQuery *query, guint index);
 
 /* metadata */
 
-GST_EXPORT
+GST_API
 void            gst_query_add_allocation_meta        (GstQuery *query, GType api, const GstStructure *params);
 
-GST_EXPORT
+GST_API
 guint           gst_query_get_n_allocation_metas     (GstQuery *query);
 
-GST_EXPORT
+GST_API
 GType           gst_query_parse_nth_allocation_meta  (GstQuery *query, guint index,
                                                       const GstStructure **params);
 
-GST_EXPORT
+GST_API
 void            gst_query_remove_nth_allocation_meta (GstQuery *query, guint index);
 
-GST_EXPORT
+GST_API
 gboolean        gst_query_find_allocation_meta       (GstQuery *query, GType api, guint *index);
 
 
@@ -569,83 +617,92 @@ typedef enum {
   GST_SCHEDULING_FLAG_BANDWIDTH_LIMITED = (1 << 2)
 } GstSchedulingFlags;
 
-GST_EXPORT
+GST_API
 GstQuery *      gst_query_new_scheduling          (void) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_query_set_scheduling          (GstQuery *query, GstSchedulingFlags flags,
                                                    gint minsize, gint maxsize, gint align);
 
-GST_EXPORT
+GST_API
 void            gst_query_parse_scheduling        (GstQuery *query, GstSchedulingFlags *flags,
                                                    gint *minsize, gint *maxsize, gint *align);
 
-GST_EXPORT
+GST_API
 void            gst_query_add_scheduling_mode       (GstQuery *query, GstPadMode mode);
 
-GST_EXPORT
+GST_API
 guint           gst_query_get_n_scheduling_modes    (GstQuery *query);
 
-GST_EXPORT
+GST_API
 GstPadMode      gst_query_parse_nth_scheduling_mode (GstQuery *query, guint index);
 
-GST_EXPORT
+GST_API
 gboolean        gst_query_has_scheduling_mode       (GstQuery *query, GstPadMode mode);
 
-GST_EXPORT
+GST_API
 gboolean        gst_query_has_scheduling_mode_with_flags (GstQuery * query, GstPadMode mode,
                                                     GstSchedulingFlags flags);
 
 /* accept-caps query */
 
-GST_EXPORT
+GST_API
 GstQuery *      gst_query_new_accept_caps          (GstCaps *caps) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_query_parse_accept_caps        (GstQuery *query, GstCaps **caps);
 
-GST_EXPORT
+GST_API
 void            gst_query_set_accept_caps_result   (GstQuery *query, gboolean result);
 
-GST_EXPORT
+GST_API
 void            gst_query_parse_accept_caps_result (GstQuery *query, gboolean *result);
 
 /* caps query */
 
-GST_EXPORT
+GST_API
 GstQuery *      gst_query_new_caps                 (GstCaps *filter) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_query_parse_caps               (GstQuery *query, GstCaps **filter);
 
-GST_EXPORT
+GST_API
 void            gst_query_set_caps_result          (GstQuery *query, GstCaps *caps);
 
-GST_EXPORT
+GST_API
 void            gst_query_parse_caps_result        (GstQuery *query, GstCaps **caps);
 
 /* drain query */
 
-GST_EXPORT
+GST_API
 GstQuery *      gst_query_new_drain                (void) G_GNUC_MALLOC;
 
 /* context query */
 
-GST_EXPORT
+GST_API
 GstQuery *      gst_query_new_context              (const gchar * context_type) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 gboolean        gst_query_parse_context_type       (GstQuery * query, const gchar ** context_type);
 
-GST_EXPORT
+GST_API
 void            gst_query_set_context              (GstQuery *query, GstContext *context);
 
-GST_EXPORT
+GST_API
 void            gst_query_parse_context            (GstQuery *query, GstContext **context);
 
-#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
+/* bitrate query */
+
+GST_API
+GstQuery *      gst_query_new_bitrate              (void) G_GNUC_MALLOC;
+
+GST_API
+void            gst_query_set_bitrate              (GstQuery * query, guint nominal_bitrate);
+
+GST_API
+void            gst_query_parse_bitrate            (GstQuery * query, guint * nominal_bitrate);
+
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstQuery, gst_query_unref)
-#endif
 
 G_END_DECLS
 
