@@ -70,7 +70,7 @@ build_field_template (GQuark field_id, const GValue * value, gpointer user_data)
   gboolean res;
 
   if (G_VALUE_TYPE (value) != GST_TYPE_STRUCTURE) {
-    GST_WARNING ("expected field of type GstStructure, but %s is %s",
+    GST_ERROR ("expected field of type GstStructure, but %s is %s",
         g_quark_to_string (field_id), G_VALUE_TYPE_NAME (value));
     return FALSE;
   }
@@ -178,7 +178,9 @@ gst_tracer_record_init (GstTracerRecord * self)
  *
  * > Please note that this is still under discussion and subject to change.
  *
- * Returns: a new #GstTracerRecord
+ * Returns: (transfer full): a new #GstTracerRecord
+ *
+ * Since: 1.8
  */
 GstTracerRecord *
 gst_tracer_record_new (const gchar * name, const gchar * firstfield, ...)
@@ -201,16 +203,22 @@ gst_tracer_record_new (const gchar * name, const gchar * firstfield, ...)
 
     /* all fields passed here must be GstStructures which we take over */
     if (type != GST_TYPE_STRUCTURE) {
-      GST_WARNING ("expected field of type GstStructure, but %s is %s",
+      GST_ERROR ("expected field of type GstStructure, but %s is %s",
           firstfield, g_type_name (type));
+      va_end (varargs);
+      gst_structure_free (structure);
+      return NULL;
     }
 
     G_VALUE_COLLECT_INIT (&val, type, varargs, G_VALUE_NOCOPY_CONTENTS, &err);
     if (G_UNLIKELY (err)) {
       g_critical ("%s", err);
+      g_free (err);
       break;
     }
-    /* see boxed_proxy_collect_value */
+    /* give ownership of the GstStructure "value" collected from varargs
+     * to this structure by unsetting the NOCOPY_CONTENTS collect-flag.
+     * see boxed_proxy_collect_value in glib's gobject/gboxed.c */
     val.data[1].v_uint &= ~G_VALUE_NOCOPY_CONTENTS;
     gst_structure_id_take_value (structure, id, &val);
 
@@ -218,7 +226,11 @@ gst_tracer_record_new (const gchar * name, const gchar * firstfield, ...)
   }
   va_end (varargs);
 
-  self = g_object_newv (GST_TYPE_TRACER_RECORD, 0, NULL);
+  self = g_object_new (GST_TYPE_TRACER_RECORD, NULL);
+
+  /* Clear floating flag */
+  gst_object_ref_sink (self);
+
   self->spec = structure;
   gst_tracer_record_build_format (self);
 
@@ -237,6 +249,8 @@ gst_tracer_record_new (const gchar * name, const gchar * firstfield, ...)
  * the category "GST_TRACER".
  *
  * > Please note that this is still under discussion and subject to change.
+ *
+ * Since: 1.8
  */
 void
 gst_tracer_record_log (GstTracerRecord * self, ...)
