@@ -892,7 +892,7 @@ file_path_contains_relatives (const gchar * path)
 
 /**
  * gst_filename_to_uri:
- * @filename: absolute or relative file name path
+ * @filename: (type filename): absolute or relative file name path
  * @error: pointer to error, or %NULL
  *
  * Similar to g_filename_to_uri(), but attempts to handle relative file paths
@@ -900,7 +900,7 @@ file_path_contains_relatives (const gchar * path)
  * the current working directory if it is a relative path, and then the path
  * will be canonicalised so that it doesn't contain any './' or '../' segments.
  *
- * On Windows #filename should be in UTF-8 encoding.
+ * On Windows @filename should be in UTF-8 encoding.
  *
  * Returns: newly-allocated URI string, or NULL on error. The caller must
  *   free the URI string with g_free() when no longer needed.
@@ -957,7 +957,7 @@ beach:
  * @short_description: URI parsing and manipulation.
  *
  * A #GstUri object can be used to parse and split a URI string into its
- * constituant parts. Two #GstUri objects can be joined to make a new #GstUri
+ * constituent parts. Two #GstUri objects can be joined to make a new #GstUri
  * using the algorithm described in RFC3986.
  */
 
@@ -1014,6 +1014,10 @@ _gst_uri_free (GstUri * uri)
     g_hash_table_unref (uri->query);
   g_free (uri->fragment);
 
+#ifdef USE_POISONING
+  memset (uri, 0xff, sizeof (*uri));
+#endif
+
   g_slice_free1 (sizeof (*uri), uri);
 }
 
@@ -2243,8 +2247,8 @@ gst_uri_set_port (GstUri * uri, guint port)
  *
  * Extract the path string from the URI object.
  *
- * Returns: (transfer full): (nullable): The path from the URI. Once finished
- *                                       with the string should be g_free()'d.
+ * Returns: (transfer full) (nullable): The path from the URI. Once finished
+ *                                      with the string should be g_free()'d.
  *
  * Since: 1.6
  */
@@ -2596,7 +2600,7 @@ gst_uri_get_query_table (const GstUri * uri)
  * reference to the new one is used instead. A value if %NULL for @query_table
  * will remove the query string from the URI.
  *
- * Returns: %TRUE if the new table was sucessfully used for the query table.
+ * Returns: %TRUE if the new table was successfully used for the query table.
  *
  * Since: 1.6
  */
@@ -2630,7 +2634,7 @@ gst_uri_set_query_table (GstUri * uri, GHashTable * query_table)
  * indicates that the key has no associated value, but will still be present in
  * the query string.
  *
- * Returns: %TRUE if the query table was sucessfully updated.
+ * Returns: %TRUE if the query table was successfully updated.
  *
  * Since: 1.6
  */
@@ -2805,7 +2809,7 @@ gst_uri_set_fragment (GstUri * uri, const gchar * fragment)
  * Get the media fragment table from the URI, as defined by "Media Fragments URI 1.0".
  * Hash table returned by this API is a list of "key-value" pairs, and the each
  * pair is generated by splitting "URI fragment" per "&" sub-delims, then "key"
- * and "value" are splitted by "=" sub-delims. The "key" returned by this API may
+ * and "value" are split by "=" sub-delims. The "key" returned by this API may
  * be undefined keyword by standard.
  * A value may be %NULL to indicate that the key should appear in the fragment
  * string in the URI, but does not have a value. Free the returned #GHashTable
