@@ -65,7 +65,7 @@ G_BEGIN_DECLS
  * GST_FOURCC_FORMAT: (skip):
  *
  * Can be used together with #GST_FOURCC_ARGS to properly output a
- * #guint32 fourcc value in a printf()-style text message.
+ * #guint32 fourcc value in a printf\()-style text message.
  *
  * |[
  * printf ("fourcc: %" GST_FOURCC_FORMAT "\n", GST_FOURCC_ARGS (fcc));
@@ -79,7 +79,7 @@ G_BEGIN_DECLS
  * @fourcc: a #guint32 fourcc value to output
  *
  * Can be used together with #GST_FOURCC_FORMAT to properly output a
- * #guint32 fourcc value in a printf()-style text message.
+ * #guint32 fourcc value in a printf\()-style text message.
  */
 
 #define __GST_PRINT_CHAR(c) \
@@ -151,7 +151,7 @@ G_BEGIN_DECLS
  *
  * Checks if the given #GValue contains a #GST_TYPE_STRUCTURE value.
  */
-#define GST_VALUE_HOLDS_STRUCTURE(x)            (G_VALUE_HOLDS((x), _gst_structure_type))
+#define GST_VALUE_HOLDS_STRUCTURE(x)         ((x) != NULL && G_VALUE_TYPE(x) == _gst_structure_type)
 
 /**
  * GST_VALUE_HOLDS_CAPS_FEATURES:
@@ -159,7 +159,7 @@ G_BEGIN_DECLS
  *
  * Checks if the given #GValue contains a #GST_TYPE_CAPS_FEATURES value.
  */
-#define GST_VALUE_HOLDS_CAPS_FEATURES(x)        (G_VALUE_HOLDS((x), _gst_caps_features_type))
+#define GST_VALUE_HOLDS_CAPS_FEATURES(x)         ((x) != NULL && G_VALUE_TYPE(x) == _gst_caps_features_type)
 
 /**
  * GST_VALUE_HOLDS_BUFFER:
@@ -209,7 +209,7 @@ G_BEGIN_DECLS
  *
  * Since: 1.6
  */
-#define GST_VALUE_HOLDS_FLAG_SET(x)     (G_TYPE_CHECK_VALUE_TYPE ((x), GST_TYPE_FLAG_SET))
+#define GST_VALUE_HOLDS_FLAG_SET(x)     (G_TYPE_FUNDAMENTAL (G_VALUE_TYPE ((x))) == GST_TYPE_FLAG_SET)
 
 /**
  * GST_FLAG_SET_MASK_EXACT: (value 4294967295) (type guint)
@@ -224,6 +224,12 @@ G_BEGIN_DECLS
 
 GST_API GType _gst_int_range_type;
 
+/**
+ * GstIntRange:
+ *
+ * A fundamental type that describes a #gint range
+ */
+
 /**
  * GST_TYPE_INT_RANGE:
  *
@@ -235,6 +241,12 @@ GST_API GType _gst_int_range_type;
 
 GST_API GType _gst_int64_range_type;
 
+/**
+ * GstInt64Range:
+ *
+ * A fundamental type that describes a #gint64 range
+ */
+
 /**
  * GST_TYPE_INT64_RANGE:
  *
@@ -246,6 +258,12 @@ GST_API GType _gst_int64_range_type;
 
 GST_API GType _gst_double_range_type;
 
+/**
+ * GstDoubleRange:
+ *
+ * A fundamental type that describes a #gdouble range
+ */
+
 /**
  * GST_TYPE_DOUBLE_RANGE:
  *
@@ -257,6 +275,12 @@ GST_API GType _gst_double_range_type;
 
 GST_API GType _gst_fraction_range_type;
 
+/**
+ * GstFractionRange:
+ *
+ * A fundamental type that describes a #GstFractionRange range
+ */
+
 /**
  * GST_TYPE_FRACTION_RANGE:
  *
@@ -268,6 +292,12 @@ GST_API GType _gst_fraction_range_type;
 
 GST_API GType _gst_value_list_type;
 
+/**
+ * GstValueList:
+ *
+ * A fundamental type that describes an unordered list of #GValue
+ */
+
 /**
  * GST_TYPE_LIST:
  *
@@ -283,6 +313,12 @@ GST_API GType _gst_value_list_type;
 
 GST_API GType _gst_value_array_type;
 
+/**
+ * GstValueArray:
+ *
+ * A fundamental type that describes an ordered list of #GValue
+ */
+
 /**
  * GST_TYPE_ARRAY:
  *
@@ -299,6 +335,13 @@ GST_API GType _gst_value_array_type;
 
 GST_API GType _gst_fraction_type;
 
+/**
+ * GstFraction:
+ *
+ * A fundamental type that describes a fraction of an integer numerator
+ * over an integer denominator
+ */
+
 /**
  * GST_TYPE_FRACTION:
  *
@@ -312,6 +355,12 @@ GST_API GType _gst_fraction_type;
 
 GST_API GType _gst_bitmask_type;
 
+/**
+ * GstBitmask:
+ *
+ * A fundamental type that describes a 64-bit bitmask
+ */
+
 /**
  * GST_TYPE_BITMASK:
  *
@@ -324,6 +373,13 @@ GST_API GType _gst_bitmask_type;
 
 GST_API GType _gst_flagset_type;
 
+/**
+ * GstFlagSet:
+ *
+ * A fundamental type that describes a 32-bit flag bitfield, with 32-bit
+ * mask indicating which of the bits in the field are explicitly set.
+ */
+
 /**
  * GST_TYPE_FLAG_SET:
  *
@@ -508,6 +564,9 @@ GST_API
 const GValue *  gst_value_list_get_value        (const GValue   *value,
                                                  guint          index);
 
+GST_API
+GValue *        gst_value_list_init             (GValue *value,
+						 guint prealloc);
 /* array */
 
 GST_API
@@ -525,6 +584,9 @@ guint           gst_value_array_get_size        (const GValue   *value);
 GST_API
 const GValue *  gst_value_array_get_value       (const GValue   *value,
                                                  guint          index);
+GST_API
+GValue *        gst_value_array_init            (GValue *value,
+						 guint prealloc);
 
 /* int range */
 
